"""
Script to create a new transaction and add
broadcast it to the testnet
:author: gurnoorsingh (20221006)
"""

import json
import base64
from algosdk.v2client import algod
from algosdk.future import transaction
import argparse

parser = argparse.ArgumentParser()
parser.add_argument(
    "--secret-key", "-s", help="Secret key of sender account", required=True
)
parser.add_argument("--sender-addr", "-a", help="Account of sender", required=True)
parser.add_argument("--algod-token", "-t", help="Token for algod", required=True)
parser.add_argument(
    "--algod-addr",
    "-d",
    help="Address of algod server",
    default="http://127.0.0.1:8080",
)
parser.add_argument("--receiver-addr", "-r", help="Account of receiver", required=True)
parser.add_argument(
    "--amount",
    "-m",
    help="Amount to be transferred in microalgos",
    default=100000,
    type=int,
)
parser.add_argument(
    "--rounds",
    "-w",
    help="Number of rounds to wait for transaction to be added",
    type=int,
    default=10,
)
args = parser.parse_args()


def new_txn(private_key, sender_address):
    algod_address = args.algod_addr
    algod_token = args.algod_token
    algod_client = algod.AlgodClient(algod_token, algod_address)

    print("My address: {}".format(sender_address))
    account_info = algod_client.account_info(sender_address)
    print("Account balance: {} microAlgos".format(account_info.get("amount")))

    # build transaction
    params = algod_client.suggested_params()

    receiver = args.receiver_addr
    amount = args.amount
    note = "Hello World".encode()

    unsigned_txn = transaction.PaymentTxn(
        sender_address, params, receiver, amount, None, note
    )

    # sign transaction
    signed_txn = unsigned_txn.sign(private_key)

    # submit transaction
    txid = algod_client.send_transaction(signed_txn)
    print("Signed transaction with txID: {}".format(txid))

    # wait for confirmation
    try:
        confirmed_txn = transaction.wait_for_confirmation(
            algod_client, txid, args.rounds
        )
    except Exception as err:
        print(err)
        return

    print("Transaction information: {}".format(json.dumps(confirmed_txn, indent=4)))
    print(
        "Decoded note: {}".format(
            base64.b64decode(confirmed_txn["txn"]["txn"]["note"]).decode()
        )
    )

    print("Starting Account balance: {} microAlgos".format(account_info.get("amount")))
    print("Amount transfered: {} microAlgos".format(amount))
    print("Fee: {} microAlgos".format(params.fee))

    account_info = algod_client.account_info(sender_address)
    print(
        "Final Account balance: {} microAlgos".format(account_info.get("amount")) + "\n"
    )


if __name__ == "__main__":
    # replace private_key and my_address with your private key and your address.
    new_txn(args.secret_key, args.sender_addr)
